use crate::{
    entity::prelude::Account,
    errors::NoUserInfo,
    handlers::{
        markups::{self, deletion_markup},
        state::NameCheckKind,
        Handler, MainDialogue, State,
    },
};
use sea_orm::prelude::*;
use teloxide::{adaptors::Throttle, prelude::*, types::ParseMode};
use tokio::task::spawn_blocking;

/// Gets the master password, decryptes the account and sends it to the user with copyable fields
async fn get_master_pass(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
    dialogue: MainDialogue,
    name: String,
    master_pass: String,
) -> crate::Result<()> {
    dialogue.exit().await?;
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let account = match Account::get(user_id, &name, &db).await? {
        Some(account) => account,
        None => {
            bot.send_message(msg.chat.id, "Account not found")
                .reply_markup(deletion_markup())
                .await?;
            return Ok(());
        }
    };
    let (login, password) = spawn_blocking(move || account.decrypt(&master_pass)).await??;
    let message = format!("Name:\n`{name}`\nLogin:\n`{login}`\nPassword:\n`{password}`");
    bot.send_message(msg.chat.id, message)
        .reply_markup(deletion_markup())
        .parse_mode(ParseMode::MarkdownV2)
        .await?;
    Ok(())
}

/// Gets the account name to get
async fn get_account_name(
    bot: Throttle<Bot>,
    msg: Message,
    _: DatabaseConnection,
    dialogue: MainDialogue,
    name: String,
) -> crate::Result<()> {
    let previous = bot
        .send_message(msg.chat.id, "Send master password")
        .await?;
    dialogue
        .update(State::GetMasterPass(Handler::new(
            move |bot, msg, db, dialogue, master_pass| {
                get_master_pass(bot, msg, db, dialogue, name, master_pass)
            },
            previous,
        )))
        .await?;
    Ok(())
}

/// Handles /get_account command
pub async fn get_account(
    bot: Throttle<Bot>,
    msg: Message,
    dialogue: MainDialogue,
    db: DatabaseConnection,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let markup = markups::account_markup(user_id, &db).await?;
    let previous = bot
        .send_message(msg.chat.id, "Send account name")
        .reply_markup(markup)
        .await?;
    dialogue
        .update(State::GetAccountName(
            Handler::new(get_account_name, previous),
            NameCheckKind::MustExist,
        ))
        .await?;
    Ok(())
}
