use crate::{entity::prelude::Account, errors::NoUserInfo, handlers::markups::deletion_markup};
use futures::TryStreamExt;
use sea_orm::prelude::*;
use std::fmt::Write;
use teloxide::{adaptors::Throttle, prelude::*, types::ParseMode};

/// Handles /get_accounts command by sending the list of copyable account names to the user
pub async fn get_accounts(
    bot: Throttle<Bot>,
    msg: Message,
    db: DatabaseConnection,
) -> crate::Result<()> {
    let user_id = msg.from().ok_or(NoUserInfo)?.id.0;
    let mut account_names = Account::get_names(user_id, &db, true).await?;
    let mut result = match account_names.try_next().await? {
        Some(name) => format!("Accounts:\n`{name}`"),
        None => {
            bot.send_message(msg.chat.id, "No accounts found")
                .reply_markup(deletion_markup())
                .await?;
            return Ok(());
        }
    };
    account_names
        .try_for_each(|name| {
            write!(result, "\n`{name}`").unwrap();
            async { Ok(()) }
        })
        .await?;
    bot.send_message(msg.chat.id, result)
        .parse_mode(ParseMode::MarkdownV2)
        .reply_markup(deletion_markup())
        .await?;
    Ok(())
}
