//! This module consists of endpoints to handle the state

mod generic;
mod get_account_name;
mod get_login;
mod get_master_pass;
mod get_password;
mod get_user;
mod handler;

pub use get_account_name::{get_account_name, NameCheckKind};
pub use get_login::get_login;
pub use get_master_pass::get_master_pass;
pub use get_password::get_password;
pub use get_user::get_user;
pub use handler::{Handler, PackagedHandler};

use crate::models::User;
use teloxide::{dispatching::dialogue::InMemStorage, prelude::*};

#[derive(Default, Clone)]
pub enum State {
    #[default]
    Start,
    GetAccountName(PackagedHandler<String>, NameCheckKind),
    GetMasterPass(PackagedHandler<String>),
    GetLogin(PackagedHandler<String>),
    GetPassword(PackagedHandler<String>),
    GetUser(PackagedHandler<User>),
}

pub type MainDialogue = Dialogue<State, InMemStorage<State>>;
